-----------------------------------
--- --- Agent-based models of Geographical Systems - chapter 28

--[[  Exploring Urban Dynamics in Latin American Cities Using an Agent-Based Simulation Approach

-- Joana Barros, Birbeck College, University of London

This model focuses on a specific kind of urban growth that happens in Latin American cities, called
‘peripherisation’. This is characterized by the formation of low-income residential
areas in the peripheral ring of the city and a perpetuation of a dynamic core-periphery
spatial pattern. 

The process of peripherisation consists of the expansion of the city
borders through the formation of peripheral low-income settlements that are then incorporated
into the city by a long-term process of expansion in which some of the lowincome
areas are recontextualised within the urban system and occupied by higher
economic groups while new low-income settlements keep emerging on the periphery.

Agents are divided into three economic groups (breeds) in a proportion based on
the division of Latin American society by income. 

The main difference betweenagents lies in their economic group. T
The simulation model is underlain by an economic logic, 
although it is not an economic model. It assumes that all agents have the same
locational preferences, which means that they all want to locate
close to the areas served by infrastructure, with nearby commerce, job opportunities
and so on. Since in Third World cities these facilities are found mostly close
to the high-income residential areas, agents look for a place close to a highincome
group residential area. The behaviours of the three income groups are
differentiated by the restrictions imposed on their economic power. Thus, the
high-income group is able to locate in any place of its preference. 
The medium-income group can locate everywhere except where the high-income group 
is already located; and, in turn the low-income group can locate only 
in otherwise vacant space. Since some agents can occupy other agents’ cells, 
this means that the latter are ‘evicted’ and must find other places to settle. 
]]--
-- Required libraries
tmepath = os.getenv("TME_PATH_1_3_0")
package.path = package.path .. ";"..tmepath.."/contrib/courselib/?.lua;./?.lua"

-- library functions that are called
local abm = require ("abm")

-- economic classes for the agents
EMPTY = 0
POOR = 1
MIDDLE = 2
RICH = 3
-- proportion of agents by economic class
proportion = { poor = 0.55, middle = 0.35, rich = 0.10 }
-- dimension of the cell space and number of agents
NDIM = 30
NAGTS = 300

-- finds a place for the agent to live
function findPlace (agent)
    --do a randomWalk
    agent:randomWalk()
    -- where am I? 
    local newcell = agent:getCell()
    -- who else is there?
    local occupants = newcell:getAgents()
    -- are there more agents there?
    if (#occupants == 1) then
        -- there is no one else; I can settle here
        newcell.color = agent.class    -- update the color of the cell
        return true
    else
        -- there is someone else that came before me
        other = occupants[1]
        -- is the previous occupant poorer than me?
        if (other.class < agent.class) then
            -- I stay, he goes
            newcell.color = agent.class -- update the color of the cell
            findPlace (other)
        else
            -- he stays, I continue to search a place for me
            findPlace (agent)
        end
    end
end
    

model = abm.Model {
    -- define a cell space
    cells = CellularSpace {
        xdim = NDIM,
        ydim = NDIM,
        neighborhood = { 
            strategy = "vonneumann",  -- (north, east, west, south)
            self     = false,
            wrap     = true
        },
        instance = { color = EMPTY }  -- initial situation of the cell space
    },
        
    -- define our agents
    family = Agent {
        init = function (self)   -- called once when the agent is created
            rnd = math.random()  -- gives a value btw 0 and 1
            if ( rnd < proportion.poor) then  -- less than 0.45
                self.class = POOR
            elseif ( rnd < (proportion.poor + proportion.middle) ) then
                self.class = MIDDLE
            else
                self.class = RICH
            end
        end
    },
    
    init =  function (model)
        -- create a society of agents 
        model.society = Society {
            instance = model.family, 
            quantity = NAGTS,
            placement = "void" ,   -- dont place the agents in space; I will do it
            max_agents_per_cell = 1
        }
        -- find the coordinates of the center of the cell space
        local center_coord = Coord { x = math.floor(NDIM/2), 
                                     y = math.floor (NDIM/2) }
        -- find the central cell
        -- agents start at the middle of the cell space to look for a place to live
        model.central_cell = model.cells:getCell (center_coord)
        
    end,
    
    changes = function (model)  
        -- for each agent, put the agent in the center cell 
        -- look for a place to live
        
        forEachAgent (model.society, 
            function (agent)
                -- enter in the central cell
                agent:enter (model.central_cell) 
                 -- find a place to live
                findPlace (agent)                
                -- notify the cell space
                model.cells:notify()
            end
        )

    end,
    -- cell space visualisation 
    maps = {
        view1 = {
            theme1 = { attribute = "color", colors = "Purples", 
                       values = { EMPTY, POOR, MIDDLE, RICH} }
                }
            }
}

model:run(1)

